"use strict";
/* eslint @typescript-eslint/no-unsafe-assignment: 0 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.UpdateRemoteBlockFromParamsJson = void 0;
/**
 * This file diffs a remote block w/ local parameters.json. It's called from blocks.ts
 * to prompt the user to confirm updating a block if any parameters have changed.
 * It's using some TypeScript type magic to ensure that all properties in parameters.json
 * are properly checked.
 */
class UpdateRemoteBlockFromParamsJson {
    constructor(config) {
        this._config = config;
    }
    // Begin transform blocks
    async getDiffedPropertiesForTransformBlock(organizationId, blockId, params) {
        const remoteBlock = (await this._config.api.organizationBlocks.getOrganizationTransformationBlock(organizationId, blockId)).transformationBlock;
        const diffCheck = this.transformBlockDiffCheck(params, remoteBlock);
        return this.getDiffedProperties(diffCheck);
    }
    async updateTransformBlock(organizationId, blockId, params) {
        const remoteBlock = (await this._config.api.organizationBlocks.getOrganizationTransformationBlock(organizationId, blockId)).transformationBlock;
        const diffCheck = this.transformBlockDiffCheck(params, remoteBlock);
        const { shouldUpdate, updateObj } = this.getUpdatedObject(diffCheck, {});
        if (shouldUpdate) {
            await this._config.api.organizationBlocks.updateOrganizationTransformationBlock(organizationId, blockId, updateObj);
        }
        return diffCheck;
    }
    transformBlockDiffCheck(params, remoteBlock) {
        let diffCheck = {
            operatesOn: { oldVal: remoteBlock.operatesOn, newVal: params.info.operatesOn },
            indMetadata: { oldVal: remoteBlock.indMetadata, newVal: params.info.indMetadata },
            cliArguments: { oldVal: remoteBlock.cliArguments, newVal: params.info.cliArguments },
            allowExtraCliArguments: {
                oldVal: remoteBlock.allowExtraCliArguments,
                newVal: params.info.allowExtraCliArguments
            },
            showInDataSources: {
                oldVal: remoteBlock.showInDataSources,
                newVal: params.info.showInDataSources
            },
            showInCreateTransformationJob: {
                oldVal: remoteBlock.showInCreateTransformationJob,
                newVal: params.info.showInCreateTransformationJob
            },
        };
        return diffCheck;
    }
    // End transform blocks
    // Begin synthetic data blocks
    async getDiffedPropertiesForSyntheticDataBlock(organizationId, blockId, params) {
        const remoteBlock = (await this._config.api.organizationBlocks.getOrganizationTransformationBlock(organizationId, blockId)).transformationBlock;
        const diffCheck = this.syntheticDataBlockDiffCheck(params, remoteBlock);
        return this.getDiffedProperties(diffCheck);
    }
    async updateSyntheticDataBlock(organizationId, blockId, params) {
        const remoteBlock = (await this._config.api.organizationBlocks.getOrganizationTransformationBlock(organizationId, blockId)).transformationBlock;
        const diffCheck = this.syntheticDataBlockDiffCheck(params, remoteBlock);
        const { shouldUpdate, updateObj } = this.getUpdatedObject(diffCheck, {});
        if (shouldUpdate) {
            await this._config.api.organizationBlocks.updateOrganizationTransformationBlock(organizationId, blockId, updateObj);
        }
        return diffCheck;
    }
    syntheticDataBlockDiffCheck(params, remoteBlock) {
        let diffCheck = {
            showInSyntheticData: { oldVal: remoteBlock.showInSyntheticData, newVal: true },
        };
        return diffCheck;
    }
    // End synthetic data blocks
    // Begin AI Action block
    async getDiffedPropertiesForAIActionBlock(organizationId, blockId, params) {
        const remoteBlock = (await this._config.api.organizationBlocks.getOrganizationTransformationBlock(organizationId, blockId)).transformationBlock;
        const diffCheck = this.aiActionBlockDiffCheck(params, remoteBlock);
        return this.getDiffedProperties(diffCheck);
    }
    async updateAIActionBlock(organizationId, blockId, params) {
        const remoteBlock = (await this._config.api.organizationBlocks.getOrganizationTransformationBlock(organizationId, blockId)).transformationBlock;
        const diffCheck = this.aiActionBlockDiffCheck(params, remoteBlock);
        const { shouldUpdate, updateObj } = this.getUpdatedObject(diffCheck, {});
        if (shouldUpdate) {
            await this._config.api.organizationBlocks.updateOrganizationTransformationBlock(organizationId, blockId, updateObj);
        }
        return diffCheck;
    }
    aiActionBlockDiffCheck(params, remoteBlock) {
        let diffCheck = {
            showInAIActions: { oldVal: remoteBlock.showInAIActions, newVal: true },
            aiActionsOperatesOn: { oldVal: remoteBlock.aiActionsOperatesOn, newVal: params.info.operatesOn },
        };
        return diffCheck;
    }
    // End synthetic data blocks
    // Begin ML blocks
    async getDiffedPropertiesForMLBlock(organizationId, blockId, params) {
        const remoteBlock = (await this._config.api.organizationBlocks.getOrganizationTransferLearningBlock(organizationId, blockId)).transferLearningBlock;
        const diffCheck = this.mlBlockDiffCheck(params, remoteBlock);
        return this.getDiffedProperties(diffCheck);
    }
    async updateMLBlock(organizationId, blockId, params) {
        const remoteBlock = (await this._config.api.organizationBlocks.getOrganizationTransferLearningBlock(organizationId, blockId)).transferLearningBlock;
        const diffCheck = this.mlBlockDiffCheck(params, remoteBlock);
        const { shouldUpdate, updateObj } = this.getUpdatedObject(diffCheck, {});
        if (shouldUpdate) {
            await this._config.api.organizationBlocks.updateOrganizationTransferLearningBlock(organizationId, blockId, updateObj);
        }
        return diffCheck;
    }
    mlBlockDiffCheck(params, remoteBlock) {
        let diffCheck = {
            operatesOn: { oldVal: remoteBlock.operatesOn, newVal: params.info.operatesOn },
            objectDetectionLastLayer: {
                oldVal: remoteBlock.objectDetectionLastLayer,
                newVal: params.info.objectDetectionLastLayer
            },
            imageInputScaling: { oldVal: remoteBlock.imageInputScaling, newVal: params.info.imageInputScaling },
            indRequiresGpu: { oldVal: remoteBlock.indRequiresGpu, newVal: params.info.indRequiresGpu },
            repositoryUrl: { oldVal: remoteBlock.repositoryUrl, newVal: params.info.repositoryUrl },
            customModelVariants: { oldVal: remoteBlock.customModelVariants, newVal: params.info.customModelVariants },
            displayCategory: { oldVal: remoteBlock.displayCategory, newVal: params.info.displayCategory },
        };
        return diffCheck;
    }
    // End ML blocks
    // Begin DSP blocks
    async getDiffedPropertiesForDSPBlock(organizationId, blockId, params) {
        const remoteBlock = (await this._config.api.organizationBlocks.getOrganizationDspBlock(organizationId, blockId)).dspBlock;
        const diffCheck = this.dspBlockDiffCheck(params, remoteBlock);
        return this.getDiffedProperties(diffCheck);
    }
    async updateDSPBlock(organizationId, blockId, params) {
        const remoteBlock = (await this._config.api.organizationBlocks.getOrganizationDspBlock(organizationId, blockId)).dspBlock;
        const diffCheck = this.dspBlockDiffCheck(params, remoteBlock);
        const { shouldUpdate, updateObj } = this.getUpdatedObject(diffCheck, {});
        if (shouldUpdate) {
            await this._config.api.organizationBlocks.updateOrganizationDspBlock(organizationId, blockId, updateObj);
        }
        return diffCheck;
    }
    dspBlockDiffCheck(params, remoteBlock) {
        let diffCheck = {
            port: { oldVal: remoteBlock.port, newVal: params.info.port },
        };
        return diffCheck;
    }
    // End DSP blocks
    // Begin Deploy blocks
    async getDiffedPropertiesForDeployBlock(organizationId, blockId, params) {
        const remoteBlock = (await this._config.api.organizationBlocks.getOrganizationDeployBlock(organizationId, blockId)).deployBlock;
        const diffCheck = this.deployBlockDiffCheck(params, remoteBlock);
        return this.getDiffedProperties(diffCheck);
    }
    async updateDeployBlock(organizationId, blockId, params) {
        const remoteBlock = (await this._config.api.organizationBlocks.getOrganizationDeployBlock(organizationId, blockId)).deployBlock;
        const diffCheck = this.deployBlockDiffCheck(params, remoteBlock);
        const { shouldUpdate, updateObj } = this.getUpdatedObject(diffCheck, {});
        if (shouldUpdate) {
            await this._config.api.organizationBlocks.updateOrganizationDeployBlock(organizationId, blockId, updateObj);
        }
        return diffCheck;
    }
    deployBlockDiffCheck(params, remoteBlock) {
        let diffCheck = {
            category: { oldVal: remoteBlock.category, newVal: params.info.category },
            integrateUrl: { oldVal: remoteBlock.integrateUrl, newVal: params.info.integrateUrl },
            cliArguments: { oldVal: remoteBlock.cliArguments, newVal: params.info.cliArguments },
            supportsEonCompiler: { oldVal: remoteBlock.supportsEonCompiler, newVal: params.info.supportsEonCompiler },
            mountLearnBlock: { oldVal: remoteBlock.mountLearnBlock, newVal: params.info.mountLearnBlock },
            showOptimizations: { oldVal: remoteBlock.showOptimizations, newVal: params.info.showOptimizations },
            privileged: { oldVal: remoteBlock.privileged, newVal: params.info.privileged },
        };
        return diffCheck;
    }
    // End Deploy blocks
    getDiffedProperties(diffCheck) {
        let ret = [];
        for (const k of Object.keys(diffCheck)) {
            let { oldVal, newVal } = diffCheck[k];
            if (JSON.stringify(oldVal) !== JSON.stringify(newVal) &&
                !(typeof oldVal !== 'undefined' && typeof newVal === 'undefined')) {
                ret.push({
                    prop: k,
                    oldVal: oldVal,
                    newVal: newVal,
                });
            }
        }
        return ret;
    }
    /**
     * Take in a models.OrganizationUpdateX object and fill it with properties
     * @param diffCheck
     * @param updateObj
     * @returns
     */
    getUpdatedObject(diffCheck, updateObj) {
        let shouldUpdate = false;
        for (const k of Object.keys(diffCheck)) {
            let { oldVal, newVal } = diffCheck[k];
            if (oldVal !== newVal && !(typeof oldVal !== 'undefined' && typeof newVal === 'undefined')) {
                updateObj[k] = newVal;
                shouldUpdate = true;
            }
        }
        return { shouldUpdate, updateObj };
    }
}
exports.UpdateRemoteBlockFromParamsJson = UpdateRemoteBlockFromParamsJson;
//# sourceMappingURL=update-remote-block-from-params-json.js.map