"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.initMicrophone = exports.initCamera = exports.CameraType = void 0;
const inquirer_1 = __importDefault(require("inquirer"));
const prophesee_1 = require("./prophesee");
const gstreamer_1 = require("./gstreamer");
const imagesnap_1 = require("./imagesnap");
const recorder_1 = require("./recorder");
var CameraType;
(function (CameraType) {
    CameraType["PropheseeCamera"] = "prophesee";
    CameraType["ImagesnapCamera"] = "imagesnap";
    CameraType["GStreamerCamera"] = "gstreamer";
    CameraType["UnknownCamera"] = "unknown";
})(CameraType = exports.CameraType || (exports.CameraType = {}));
async function initCamera(cameraType, cameraDeviceName, dimensions, gstLaunchArgs, verboseOutput) {
    let camera;
    if (cameraType === CameraType.PropheseeCamera) {
        camera = new prophesee_1.Prophesee(verboseOutput);
    }
    else if (cameraType === CameraType.ImagesnapCamera) {
        camera = new imagesnap_1.Imagesnap(verboseOutput);
    }
    else if (cameraType === CameraType.GStreamerCamera) {
        camera = new gstreamer_1.GStreamer(verboseOutput, {
            customLaunchCommand: gstLaunchArgs,
        });
    }
    else {
        throw new Error('Unsupported camera type "' + cameraType + '"');
    }
    await camera.init();
    let cameraDevice;
    const cameraDevices = await camera.listDevices();
    if (cameraDevices.length === 0) {
        throw new Error('Cannot find any webcams');
    }
    if (cameraDeviceName && cameraDevices.find(d => d === cameraDeviceName)) {
        cameraDevice = cameraDeviceName;
    }
    else if (cameraDevices.length === 1) {
        cameraDevice = cameraDevices[0];
    }
    else {
        let inqRes = await inquirer_1.default.prompt([{
                type: 'list',
                choices: (cameraDevices || []).map(p => ({ name: p, value: p })),
                name: 'camera',
                message: 'Select a camera',
                pageSize: 20
            }]);
        cameraDevice = inqRes.camera;
    }
    //TODO: move this message out of the camera init function
    // console.log(RUNNER_PREFIX, 'Using camera', cameraDevice, 'starting...');
    if (cameraType === CameraType.PropheseeCamera) {
        await camera.start({
            device: cameraDevice,
            intervalMs: 40,
            dimensions: dimensions
        });
    }
    else {
        await camera.start({
            device: cameraDevice,
            intervalMs: 100,
            dimensions: dimensions
        });
    }
    return camera;
}
exports.initCamera = initCamera;
async function initMicrophone(audioDeviceName) {
    let audioDevice;
    const audioDevices = await recorder_1.AudioRecorder.ListDevices();
    if (audioDeviceName && audioDevices.find(d => d.id === audioDeviceName)) {
        audioDevice = audioDeviceName;
    }
    else if (audioDevices.length === 1) {
        audioDevice = audioDevices[0].id;
    }
    else if (audioDevices.length === 0) {
        throw Error('Could not find any microphones');
    }
    else {
        let inqRes = await inquirer_1.default.prompt([{
                type: 'list',
                choices: (audioDevices || []).map(p => ({ name: p.name, value: p.id })),
                name: 'microphone',
                message: 'Select a microphone',
                pageSize: 20
            }]);
        audioDevice = inqRes.microphone;
    }
    return audioDevice;
}
exports.initMicrophone = initMicrophone;
//# sourceMappingURL=sensors-helper.js.map