"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const block_config_manager_1 = require("../../cli/blocks/block-config-manager");
const os_1 = __importDefault(require("os"));
describe("block config migration (deploy)", () => {
    describe('deploy', () => {
        it('converts v1 deploy block (other host, no parameters.json)', async () => {
            let config = {};
            config.host = '211.212.213.214';
            let folder = await fs_1.default.promises.mkdtemp(path_1.default.join(os_1.default.tmpdir() + '-ei-'));
            let eiBlockConfig = {
                "version": 1,
                "config": {
                    "edgeimpulse.com": {
                        "name": "Micro:bit 2",
                        "type": "deploy",
                        "description": "Firmware with microphone driver for the micro:bit",
                        "deployCategory": "firmware",
                        "organizationId": 1,
                        "id": 200
                    }
                }
            };
            await fs_1.default.promises.writeFile(path_1.default.join(folder, '.ei-block-config'), JSON.stringify(eiBlockConfig, null, 4), 'utf-8');
            const blockConfigManager = new block_config_manager_1.BlockConfigManager(config, folder, {
                skipConfirmation: true,
            });
            const blockConfig = await blockConfigManager.loadConfig({ throwOnMissingParams: false });
            (0, assert_1.default)(blockConfig !== null, 'blockConfig should not be null');
            if (blockConfig.type !== 'deploy') {
                (0, assert_1.default)(false, 'blockConfig type should be "deploy": ' + JSON.stringify(blockConfig));
            }
            assert_1.default.deepEqual(blockConfig.type, 'deploy');
            assert_1.default.deepEqual(blockConfig.config, null); // host is different
            assert_1.default.deepEqual(blockConfig.parameters, {
                "version": 1,
                "type": "deploy",
                "info": {
                    "name": "Micro:bit 2",
                    "description": "Firmware with microphone driver for the micro:bit",
                    "category": "firmware",
                    "cliArguments": '',
                    "mountLearnBlock": false,
                    "showOptimizations": true,
                    "supportsEonCompiler": true,
                },
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, '.ei-block-config'), 'utf-8')), {
                "version": 2,
                "config": {
                    "edgeimpulse.com": {
                        "organizationId": 1,
                        "id": 200
                    }
                }
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, 'parameters.json'), 'utf-8')), {
                "version": 1,
                "type": "deploy",
                "info": {
                    "name": "Micro:bit 2",
                    "description": "Firmware with microphone driver for the micro:bit",
                    "category": "firmware",
                    "cliArguments": '',
                    "mountLearnBlock": false,
                    "showOptimizations": true,
                    "supportsEonCompiler": true,
                },
            });
        });
        it('converts v1 deploy block (same host, no parameters.json)', async () => {
            let config = {};
            config.host = '211.212.213.214';
            config.api = {
                organizationBlocks: {
                    getOrganizationDeployBlock: async (organizationId, blockId) => {
                        return {
                            success: true,
                            deployBlock: {
                                id: blockId,
                                cliArguments: '--test',
                                mountLearnBlock: false,
                                showOptimizations: true,
                                supportsEonCompiler: true,
                            }
                        };
                    },
                },
            };
            let folder = await fs_1.default.promises.mkdtemp(path_1.default.join(os_1.default.tmpdir() + '-ei-'));
            let eiBlockConfig = {
                "version": 1,
                "config": {
                    "211.212.213.214": {
                        "name": "Micro:bit 2",
                        "type": "deploy",
                        "description": "Firmware with microphone driver for the micro:bit",
                        "organizationId": 1,
                        "deployCategory": "firmware",
                        "id": 200
                    }
                }
            };
            await fs_1.default.promises.writeFile(path_1.default.join(folder, '.ei-block-config'), JSON.stringify(eiBlockConfig, null, 4), 'utf-8');
            const blockConfigManager = new block_config_manager_1.BlockConfigManager(config, folder, {
                skipConfirmation: true,
            });
            const blockConfig = await blockConfigManager.loadConfig({ throwOnMissingParams: false });
            (0, assert_1.default)(blockConfig !== null, 'blockConfig should not be null');
            if (blockConfig.type !== 'deploy') {
                (0, assert_1.default)(false, 'blockConfig type should be "deploy": ' + JSON.stringify(blockConfig));
            }
            assert_1.default.deepEqual(blockConfig.type, 'deploy');
            assert_1.default.deepEqual(blockConfig.config, {
                "organizationId": 1,
                "id": 200
            });
            assert_1.default.deepEqual(blockConfig.parameters, {
                "version": 1,
                "type": "deploy",
                "info": {
                    "name": "Micro:bit 2",
                    "description": "Firmware with microphone driver for the micro:bit",
                    "category": "firmware",
                    "cliArguments": '--test',
                    "mountLearnBlock": false,
                    "showOptimizations": true,
                    "supportsEonCompiler": true,
                },
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, '.ei-block-config'), 'utf-8')), {
                "version": 2,
                "config": {
                    "211.212.213.214": {
                        "organizationId": 1,
                        "id": 200
                    }
                }
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, 'parameters.json'), 'utf-8')), {
                "version": 1,
                "type": "deploy",
                "info": {
                    "name": "Micro:bit 2",
                    "description": "Firmware with microphone driver for the micro:bit",
                    "category": "firmware",
                    "cliArguments": '--test',
                    "mountLearnBlock": false,
                    "showOptimizations": true,
                    "supportsEonCompiler": true,
                },
            });
        });
        it('loads v2 deploy block (same host, new parameters.json)', async () => {
            let config = {};
            config.host = '211.212.213.214';
            let folder = await fs_1.default.promises.mkdtemp(path_1.default.join(os_1.default.tmpdir() + '-ei-'));
            let eiBlockConfig = {
                "version": 2,
                "config": {
                    "211.212.213.214": {
                        "organizationId": 1,
                        "id": 200
                    },
                }
            };
            await fs_1.default.promises.writeFile(path_1.default.join(folder, '.ei-block-config'), JSON.stringify(eiBlockConfig, null, 4), 'utf-8');
            await fs_1.default.promises.writeFile(path_1.default.join(folder, 'parameters.json'), JSON.stringify({
                "version": 1,
                "type": "deploy",
                "info": {
                    "name": "My awesome block",
                    "description": "Don't overwrite dis",
                    "category": "library",
                },
            }, null, 4), 'utf-8');
            const blockConfigManager = new block_config_manager_1.BlockConfigManager(config, folder, {
                skipConfirmation: true,
            });
            const blockConfig = await blockConfigManager.loadConfig({ throwOnMissingParams: false });
            (0, assert_1.default)(blockConfig !== null, 'blockConfig should not be null');
            if (blockConfig.type !== 'deploy') {
                (0, assert_1.default)(false, 'blockConfig type should be "deploy": ' + JSON.stringify(blockConfig));
            }
            assert_1.default.deepEqual(blockConfig.type, 'deploy');
            assert_1.default.deepEqual(blockConfig.config, {
                "organizationId": 1,
                "id": 200
            });
            assert_1.default.deepEqual(blockConfig.parameters, {
                "version": 1,
                "type": "deploy",
                "info": {
                    "name": "My awesome block",
                    "description": "Don't overwrite dis",
                    "category": "library",
                },
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, '.ei-block-config'), 'utf-8')), {
                "version": 2,
                "config": {
                    "211.212.213.214": {
                        "organizationId": 1,
                        "id": 200
                    }
                }
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, 'parameters.json'), 'utf-8')), {
                "version": 1,
                "type": "deploy",
                "info": {
                    "name": "My awesome block",
                    "description": "Don't overwrite dis",
                    "category": "library",
                },
            });
        });
    });
});
//# sourceMappingURL=deploy.test.js.map