"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const block_config_manager_1 = require("../../cli/blocks/block-config-manager");
const os_1 = __importDefault(require("os"));
describe("block config migration (dsp)", () => {
    describe('dsp', () => {
        it('throws error if parameters.json missing', async () => {
            let config = {};
            config.host = '211.212.213.214';
            let folder = await fs_1.default.promises.mkdtemp(path_1.default.join(os_1.default.tmpdir() + '-ei-'));
            let eiBlockConfig = {
                "version": 1,
                "config": {
                    "edgeimpulse.com": {
                        "name": "Pose estimation",
                        "id": 6,
                        "type": "dsp",
                        "description": "Test 123",
                        "organizationId": 1,
                        "tlIndRequiresGpu": false,
                        "port": 4446,
                    },
                    "edgeimpulse2.com": {
                        "name": "Pose estimation",
                        "id": 7,
                        "type": "dsp",
                        "description": "Test 123",
                        "organizationId": 2,
                        "tlIndRequiresGpu": false,
                    }
                }
            };
            await fs_1.default.promises.writeFile(path_1.default.join(folder, '.ei-block-config'), JSON.stringify(eiBlockConfig, null, 4), 'utf-8');
            let errorMsg;
            try {
                const blockConfigManager = new block_config_manager_1.BlockConfigManager(config, folder, {
                    skipConfirmation: true,
                });
                await blockConfigManager.loadConfig({ throwOnMissingParams: false });
            }
            catch (ex2) {
                let ex = ex2;
                errorMsg = ex.message || ex.toString();
            }
            (0, assert_1.default)(errorMsg, `Expected an error to be thrown`);
            (0, assert_1.default)(errorMsg.indexOf(`You're missing a "parameters.json" file`) !== -1, `error message should have 'You're missing a "parameters.json" file' (${errorMsg})`);
        });
        it('converts v1 dsp block (old parameters.json)', async () => {
            let config = {};
            config.host = '211.212.213.214';
            let folder = await fs_1.default.promises.mkdtemp(path_1.default.join(os_1.default.tmpdir() + '-ei-'));
            let eiBlockConfig = {
                "version": 1,
                "config": {
                    "211.212.213.214": {
                        "name": "Pose estimation",
                        "id": 6,
                        "type": "dsp",
                        "description": "Test 123",
                        "organizationId": 1,
                        "tlIndRequiresGpu": false,
                        "port": 4446,
                    }
                }
            };
            await fs_1.default.promises.writeFile(path_1.default.join(folder, '.ei-block-config'), JSON.stringify(eiBlockConfig, null, 4), 'utf-8');
            await fs_1.default.promises.writeFile(path_1.default.join(folder, 'parameters.json'), JSON.stringify({
                "info": {
                    "title": "Audio (MFE)",
                    "author": "Edge Impulse",
                    "description": "Extracts a spectrogram from audio signals using Mel-filterbank energy features, great for non-voice audio.",
                    "name": "MFE",
                    "preferConvolution": true,
                    "convolutionColumns": "num_filters",
                    "convolutionKernelSize": 5,
                    "cppType": "mfe",
                    "visualization": "dimensionalityReduction",
                    "experimental": false,
                    "latestImplementationVersion": 4,
                    "hasAutoTune": true
                },
                "parameters": [{
                        "name": "Window size",
                        "value": 101,
                        "type": "int",
                        "help": "The size of sliding window for local normalization",
                        "param": "win_size",
                        "showForImplementationVersion": [1, 2]
                    }]
            }, null, 4), 'utf-8');
            const blockConfigManager = new block_config_manager_1.BlockConfigManager(config, folder, {
                skipConfirmation: true,
            });
            const blockConfig = await blockConfigManager.loadConfig({ throwOnMissingParams: false });
            (0, assert_1.default)(blockConfig !== null, 'blockConfig should not be null');
            if (blockConfig.type !== 'dsp') {
                (0, assert_1.default)(false, 'blockConfig type should be "dsp": ' + JSON.stringify(blockConfig));
            }
            assert_1.default.deepEqual(blockConfig.type, 'dsp');
            assert_1.default.deepEqual(blockConfig.config, {
                "organizationId": 1,
                "id": 6
            });
            assert_1.default.deepEqual(blockConfig.parameters, {
                "version": 1,
                "type": "dsp",
                "info": {
                    "title": "Audio (MFE)",
                    "author": "Edge Impulse",
                    "description": "Extracts a spectrogram from audio signals using Mel-filterbank energy features, great for non-voice audio.",
                    "name": "MFE",
                    "preferConvolution": true,
                    "convolutionColumns": "num_filters",
                    "convolutionKernelSize": 5,
                    "cppType": "mfe",
                    "visualization": "dimensionalityReduction",
                    "experimental": false,
                    "latestImplementationVersion": 4,
                    "hasAutoTune": true,
                    "port": 4446, // <-- this should have been added
                },
                "parameters": [
                    {
                        "name": "Window size",
                        "value": 101,
                        "type": "int",
                        "help": "The size of sliding window for local normalization",
                        "param": "win_size",
                        "showForImplementationVersion": [1, 2]
                    }
                ],
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, '.ei-block-config'), 'utf-8')), {
                "version": 2,
                "config": {
                    "211.212.213.214": {
                        "organizationId": 1,
                        "id": 6
                    }
                }
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, 'parameters.json'), 'utf-8')), {
                "version": 1,
                "type": "dsp",
                "info": {
                    "title": "Audio (MFE)",
                    "author": "Edge Impulse",
                    "description": "Extracts a spectrogram from audio signals using Mel-filterbank energy features, great for non-voice audio.",
                    "name": "MFE",
                    "preferConvolution": true,
                    "convolutionColumns": "num_filters",
                    "convolutionKernelSize": 5,
                    "cppType": "mfe",
                    "visualization": "dimensionalityReduction",
                    "experimental": false,
                    "latestImplementationVersion": 4,
                    "hasAutoTune": true,
                    "port": 4446, // <-- this should have been added
                },
                "parameters": [
                    {
                        "name": "Window size",
                        "value": 101,
                        "type": "int",
                        "help": "The size of sliding window for local normalization",
                        "param": "win_size",
                        "showForImplementationVersion": [1, 2]
                    },
                ],
            });
        });
        it('converts v1 dsp block (new parameters.json)', async () => {
            let config = {};
            config.host = '211.212.213.214';
            let folder = await fs_1.default.promises.mkdtemp(path_1.default.join(os_1.default.tmpdir() + '-ei-'));
            let eiBlockConfig = {
                "version": 1,
                "config": {
                    "211.212.213.214": {
                        "name": "Pose estimation",
                        "id": 6,
                        "type": "dsp",
                        "description": "Test 123",
                        "organizationId": 1,
                        "tlIndRequiresGpu": false,
                        "port": 4446,
                    }
                }
            };
            await fs_1.default.promises.writeFile(path_1.default.join(folder, '.ei-block-config'), JSON.stringify(eiBlockConfig, null, 4), 'utf-8');
            await fs_1.default.promises.writeFile(path_1.default.join(folder, 'parameters.json'), JSON.stringify({
                "version": 1,
                "type": "dsp",
                "info": {
                    "name": "My awesome block",
                    "description": "Don't overwrite dis",
                    "port": 5100, // <-- should not be overwritten
                },
                "parameters": [
                    {
                        "name": "Window size",
                        "value": 101,
                        "type": "int",
                        "help": "The size of sliding window for local normalization",
                        "param": "win_size",
                        "showForImplementationVersion": [1, 2]
                    },
                ]
            }, null, 4), 'utf-8');
            const blockConfigManager = new block_config_manager_1.BlockConfigManager(config, folder, {
                skipConfirmation: true,
            });
            const blockConfig = await blockConfigManager.loadConfig({ throwOnMissingParams: false });
            (0, assert_1.default)(blockConfig !== null, 'blockConfig should not be null');
            if (blockConfig.type !== 'dsp') {
                (0, assert_1.default)(false, 'blockConfig type should be "dsp": ' + JSON.stringify(blockConfig));
            }
            assert_1.default.deepEqual(blockConfig.type, 'dsp');
            assert_1.default.deepEqual(blockConfig.config, {
                "organizationId": 1,
                "id": 6
            });
            assert_1.default.deepEqual(blockConfig.parameters, {
                "version": 1,
                "type": "dsp",
                "info": {
                    "name": "My awesome block",
                    "description": "Don't overwrite dis",
                    "port": 5100, // <-- should not be overwritten
                },
                "parameters": [
                    {
                        "name": "Window size",
                        "value": 101,
                        "type": "int",
                        "help": "The size of sliding window for local normalization",
                        "param": "win_size",
                        "showForImplementationVersion": [1, 2]
                    }
                ],
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, '.ei-block-config'), 'utf-8')), {
                "version": 2,
                "config": {
                    "211.212.213.214": {
                        "organizationId": 1,
                        "id": 6
                    }
                }
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, 'parameters.json'), 'utf-8')), {
                "version": 1,
                "type": "dsp",
                "info": {
                    "name": "My awesome block",
                    "description": "Don't overwrite dis",
                    "port": 5100,
                },
                "parameters": [
                    {
                        "name": "Window size",
                        "value": 101,
                        "type": "int",
                        "help": "The size of sliding window for local normalization",
                        "param": "win_size",
                        "showForImplementationVersion": [1, 2]
                    },
                ],
            });
        });
        it('loads v2 dsp block (new parameters.json)', async () => {
            let config = {};
            config.host = '211.212.213.214';
            let folder = await fs_1.default.promises.mkdtemp(path_1.default.join(os_1.default.tmpdir() + '-ei-'));
            let eiBlockConfig = {
                "version": 2,
                "config": {
                    "211.212.213.214": {
                        "organizationId": 1,
                        "id": 6
                    }
                }
            };
            await fs_1.default.promises.writeFile(path_1.default.join(folder, '.ei-block-config'), JSON.stringify(eiBlockConfig, null, 4), 'utf-8');
            await fs_1.default.promises.writeFile(path_1.default.join(folder, 'parameters.json'), JSON.stringify({
                "version": 1,
                "type": "dsp",
                "info": {
                    "name": "My awesome block",
                    "description": "Don't overwrite dis",
                    "port": 5100,
                },
                "parameters": [
                    {
                        "name": "OpenAI API Key",
                        "value": "",
                        "type": "secret",
                        "help": "An API Key that gives access to OpenAI",
                        "param": "OPENAI_API_KEY"
                    },
                ]
            }, null, 4), 'utf-8');
            const blockConfigManager = new block_config_manager_1.BlockConfigManager(config, folder, {
                skipConfirmation: true,
            });
            const blockConfig = await blockConfigManager.loadConfig({ throwOnMissingParams: false });
            (0, assert_1.default)(blockConfig !== null, 'blockConfig should not be null');
            if (blockConfig.type !== 'dsp') {
                (0, assert_1.default)(false, 'blockConfig type should be "dsp": ' + JSON.stringify(blockConfig));
            }
            assert_1.default.deepEqual(blockConfig.type, 'dsp');
            assert_1.default.deepEqual(blockConfig.config, {
                "organizationId": 1,
                "id": 6
            });
            assert_1.default.deepEqual(blockConfig.parameters, {
                "version": 1,
                "type": "dsp",
                "info": {
                    "name": "My awesome block",
                    "description": "Don't overwrite dis",
                    "port": 5100,
                },
                "parameters": [
                    {
                        "name": "OpenAI API Key",
                        "value": "",
                        "type": "secret",
                        "help": "An API Key that gives access to OpenAI",
                        "param": "OPENAI_API_KEY"
                    }
                ],
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, '.ei-block-config'), 'utf-8')), {
                "version": 2,
                "config": {
                    "211.212.213.214": {
                        "organizationId": 1,
                        "id": 6
                    }
                }
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, 'parameters.json'), 'utf-8')), {
                "version": 1,
                "type": "dsp",
                "info": {
                    "name": "My awesome block",
                    "description": "Don't overwrite dis",
                    "port": 5100,
                },
                "parameters": [
                    {
                        "name": "OpenAI API Key",
                        "value": "",
                        "type": "secret",
                        "help": "An API Key that gives access to OpenAI",
                        "param": "OPENAI_API_KEY"
                    },
                ],
            });
        });
    });
});
//# sourceMappingURL=dsp.test.js.map