"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const block_config_manager_1 = require("../../cli/blocks/block-config-manager");
const os_1 = __importDefault(require("os"));
describe("block config migration (transform)", () => {
    describe('synthetic data', () => {
        it('converts v1 synthetic data (other host, no parameters.json)', async () => {
            let config = {};
            config.host = '211.212.213.214';
            let folder = await fs_1.default.promises.mkdtemp(path_1.default.join(os_1.default.tmpdir() + '-ei-'));
            let eiBlockConfig = {
                "version": 1,
                "config": {
                    "edgeimpulse.com": {
                        "name": "ElevenLabs Synthetic Audio Generator",
                        "type": "transform",
                        "description": "Uses ElevenLabs (https://elevenlabs.io/) text-to-audio model to generate synthetic non-voice audio samples. Very useful for building audible even detection on synthetic data.",
                        "organizationId": 1,
                        "operatesOn": "standalone",
                        "tlIndRequiresGpu": false,
                        "transformMountpoints": [],
                        "id": 7174
                    },
                    "edgeimpulse2.com": {
                        "name": "ElevenLabs Synthetic Audio Generator",
                        "type": "transform",
                        "description": "Uses ElevenLabs (https://elevenlabs.io/) text-to-audio model to generate synthetic non-voice audio samples. Very useful for building audible even detection on synthetic data.",
                        "organizationId": 2,
                        "operatesOn": "standalone",
                        "tlIndRequiresGpu": false,
                        "transformMountpoints": [],
                        "id": 7175
                    }
                }
            };
            await fs_1.default.promises.writeFile(path_1.default.join(folder, '.ei-block-config'), JSON.stringify(eiBlockConfig, null, 4), 'utf-8');
            const blockConfigManager = new block_config_manager_1.BlockConfigManager(config, folder, {
                skipConfirmation: true,
                transformWhatTypeOfBlockIsThisReply: 'synthetic-data',
            });
            const blockConfig = await blockConfigManager.loadConfig({ throwOnMissingParams: false });
            (0, assert_1.default)(blockConfig !== null, 'blockConfig should not be null');
            if (blockConfig.type !== 'synthetic-data') {
                (0, assert_1.default)(false, 'blockConfig type should be "synthetic-data": ' + JSON.stringify(blockConfig));
            }
            assert_1.default.deepEqual(blockConfig.type, 'synthetic-data');
            assert_1.default.deepEqual(blockConfig.config, null); // host is different
            assert_1.default.deepEqual(blockConfig.parameters, {
                "version": 1,
                "type": "synthetic-data",
                "info": {
                    "name": "ElevenLabs Synthetic Audio Generator",
                    "description": "Uses ElevenLabs (https://elevenlabs.io/) text-to-audio model to generate synthetic non-voice audio samples. Very useful for building audible even detection on synthetic data."
                },
                "parameters": []
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, '.ei-block-config'), 'utf-8')), {
                "version": 2,
                "config": {
                    "edgeimpulse.com": {
                        "organizationId": 1,
                        "id": 7174
                    },
                    "edgeimpulse2.com": {
                        "organizationId": 2,
                        "id": 7175
                    }
                }
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, 'parameters.json'), 'utf-8')), {
                "version": 1,
                "type": "synthetic-data",
                "info": {
                    "name": "ElevenLabs Synthetic Audio Generator",
                    "description": "Uses ElevenLabs (https://elevenlabs.io/) text-to-audio model to generate synthetic non-voice audio samples. Very useful for building audible even detection on synthetic data."
                },
                "parameters": []
            });
        });
        it('converts v1 synthetic data (same host, no parameters.json)', async () => {
            let config = {};
            config.host = '211.212.213.214';
            config.api = {
                organizationBlocks: {
                    getOrganizationTransformationBlock: async (organizationId, blockId) => {
                        return {
                            success: true,
                            transformationBlock: {
                                id: blockId,
                                showInSyntheticData: true,
                            }
                        };
                    },
                },
            };
            let folder = await fs_1.default.promises.mkdtemp(path_1.default.join(os_1.default.tmpdir() + '-ei-'));
            let eiBlockConfig = {
                "version": 1,
                "config": {
                    "211.212.213.214": {
                        "name": "ElevenLabs Synthetic Audio Generator",
                        "type": "transform",
                        "description": "Uses ElevenLabs (https://elevenlabs.io/) text-to-audio model to generate synthetic non-voice audio samples. Very useful for building audible even detection on synthetic data.",
                        "organizationId": 1,
                        "operatesOn": "standalone",
                        "tlIndRequiresGpu": false,
                        "transformMountpoints": [],
                        "id": 7174
                    }
                }
            };
            await fs_1.default.promises.writeFile(path_1.default.join(folder, '.ei-block-config'), JSON.stringify(eiBlockConfig, null, 4), 'utf-8');
            const blockConfigManager = new block_config_manager_1.BlockConfigManager(config, folder, {
                skipConfirmation: true,
            });
            const blockConfig = await blockConfigManager.loadConfig({ throwOnMissingParams: false });
            (0, assert_1.default)(blockConfig !== null, 'blockConfig should not be null');
            if (blockConfig.type !== 'synthetic-data') {
                (0, assert_1.default)(false, 'blockConfig type should be "synthetic-data": ' + JSON.stringify(blockConfig));
            }
            assert_1.default.deepEqual(blockConfig.type, 'synthetic-data');
            assert_1.default.deepEqual(blockConfig.config, {
                "organizationId": 1,
                "id": 7174
            });
            assert_1.default.deepEqual(blockConfig.parameters, {
                "version": 1,
                "type": "synthetic-data",
                "info": {
                    "name": "ElevenLabs Synthetic Audio Generator",
                    "description": "Uses ElevenLabs (https://elevenlabs.io/) text-to-audio model to generate synthetic non-voice audio samples. Very useful for building audible even detection on synthetic data."
                },
                "parameters": []
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, '.ei-block-config'), 'utf-8')), {
                "version": 2,
                "config": {
                    "211.212.213.214": {
                        "organizationId": 1,
                        "id": 7174
                    }
                }
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, 'parameters.json'), 'utf-8')), {
                "version": 1,
                "type": "synthetic-data",
                "info": {
                    "name": "ElevenLabs Synthetic Audio Generator",
                    "description": "Uses ElevenLabs (https://elevenlabs.io/) text-to-audio model to generate synthetic non-voice audio samples. Very useful for building audible even detection on synthetic data."
                },
                "parameters": []
            });
        });
        it('converts v1 synthetic data (same host, old parameters.json)', async () => {
            let config = {};
            config.host = '211.212.213.214';
            config.api = {
                organizationBlocks: {
                    getOrganizationTransformationBlock: async (organizationId, blockId) => {
                        return {
                            success: true,
                            transformationBlock: {
                                id: blockId,
                                showInSyntheticData: true,
                            }
                        };
                    },
                },
            };
            let folder = await fs_1.default.promises.mkdtemp(path_1.default.join(os_1.default.tmpdir() + '-ei-'));
            let eiBlockConfig = {
                "version": 1,
                "config": {
                    "211.212.213.214": {
                        "name": "ElevenLabs Synthetic Audio Generator",
                        "type": "transform",
                        "description": "Uses ElevenLabs (https://elevenlabs.io/) text-to-audio model to generate synthetic non-voice audio samples. Very useful for building audible even detection on synthetic data.",
                        "organizationId": 1,
                        "operatesOn": "standalone",
                        "tlIndRequiresGpu": false,
                        "transformMountpoints": [],
                        "id": 7174
                    }
                }
            };
            await fs_1.default.promises.writeFile(path_1.default.join(folder, '.ei-block-config'), JSON.stringify(eiBlockConfig, null, 4), 'utf-8');
            await fs_1.default.promises.writeFile(path_1.default.join(folder, 'parameters.json'), JSON.stringify([
                {
                    "name": "OpenAI API Key",
                    "value": "",
                    "type": "secret",
                    "help": "An API Key that gives access to OpenAI",
                    "param": "OPENAI_API_KEY"
                }
            ], null, 4), 'utf-8');
            const blockConfigManager = new block_config_manager_1.BlockConfigManager(config, folder, {
                skipConfirmation: true,
            });
            const blockConfig = await blockConfigManager.loadConfig({ throwOnMissingParams: false });
            (0, assert_1.default)(blockConfig !== null, 'blockConfig should not be null');
            if (blockConfig.type !== 'synthetic-data') {
                (0, assert_1.default)(false, 'blockConfig type should be "synthetic-data": ' + JSON.stringify(blockConfig));
            }
            assert_1.default.deepEqual(blockConfig.type, 'synthetic-data');
            assert_1.default.deepEqual(blockConfig.config, {
                "organizationId": 1,
                "id": 7174
            });
            assert_1.default.deepEqual(blockConfig.parameters, {
                "version": 1,
                "type": "synthetic-data",
                "info": {
                    "name": "ElevenLabs Synthetic Audio Generator",
                    "description": "Uses ElevenLabs (https://elevenlabs.io/) text-to-audio model to generate synthetic non-voice audio samples. Very useful for building audible even detection on synthetic data."
                },
                "parameters": [
                    {
                        "name": "OpenAI API Key",
                        "value": "",
                        "type": "secret",
                        "help": "An API Key that gives access to OpenAI",
                        "param": "OPENAI_API_KEY"
                    }
                ],
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, '.ei-block-config'), 'utf-8')), {
                "version": 2,
                "config": {
                    "211.212.213.214": {
                        "organizationId": 1,
                        "id": 7174
                    }
                }
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, 'parameters.json'), 'utf-8')), {
                "version": 1,
                "type": "synthetic-data",
                "info": {
                    "name": "ElevenLabs Synthetic Audio Generator",
                    "description": "Uses ElevenLabs (https://elevenlabs.io/) text-to-audio model to generate synthetic non-voice audio samples. Very useful for building audible even detection on synthetic data."
                },
                "parameters": [
                    {
                        "name": "OpenAI API Key",
                        "value": "",
                        "type": "secret",
                        "help": "An API Key that gives access to OpenAI",
                        "param": "OPENAI_API_KEY"
                    },
                ],
            });
        });
        it('converts v1 synthetic data (same host, new parameters.json)', async () => {
            let config = {};
            config.host = '211.212.213.214';
            config.api = {
                organizationBlocks: {
                    getOrganizationTransformationBlock: async (organizationId, blockId) => {
                        return {
                            success: true,
                            transformationBlock: {
                                id: blockId,
                                showInSyntheticData: true,
                            }
                        };
                    },
                },
            };
            let folder = await fs_1.default.promises.mkdtemp(path_1.default.join(os_1.default.tmpdir() + '-ei-'));
            let eiBlockConfig = {
                "version": 1,
                "config": {
                    "211.212.213.214": {
                        "name": "ElevenLabs Synthetic Audio Generator",
                        "type": "transform",
                        "description": "Uses ElevenLabs (https://elevenlabs.io/) text-to-audio model to generate synthetic non-voice audio samples. Very useful for building audible even detection on synthetic data.",
                        "organizationId": 1,
                        "operatesOn": "standalone",
                        "tlIndRequiresGpu": false,
                        "transformMountpoints": [],
                        "id": 7174
                    }
                }
            };
            await fs_1.default.promises.writeFile(path_1.default.join(folder, '.ei-block-config'), JSON.stringify(eiBlockConfig, null, 4), 'utf-8');
            await fs_1.default.promises.writeFile(path_1.default.join(folder, 'parameters.json'), JSON.stringify({
                "version": 1,
                "type": "synthetic-data",
                "info": {
                    "name": "My awesome block",
                    "description": "Don't overwrite dis"
                },
                "parameters": [
                    {
                        "name": "OpenAI API Key",
                        "value": "",
                        "type": "secret",
                        "help": "An API Key that gives access to OpenAI",
                        "param": "OPENAI_API_KEY"
                    },
                ]
            }, null, 4), 'utf-8');
            const blockConfigManager = new block_config_manager_1.BlockConfigManager(config, folder, {
                skipConfirmation: true,
            });
            const blockConfig = await blockConfigManager.loadConfig({ throwOnMissingParams: false });
            (0, assert_1.default)(blockConfig !== null, 'blockConfig should not be null');
            if (blockConfig.type !== 'synthetic-data') {
                (0, assert_1.default)(false, 'blockConfig type should be "synthetic-data": ' + JSON.stringify(blockConfig));
            }
            assert_1.default.deepEqual(blockConfig.type, 'synthetic-data');
            assert_1.default.deepEqual(blockConfig.config, {
                "organizationId": 1,
                "id": 7174
            });
            assert_1.default.deepEqual(blockConfig.parameters, {
                "version": 1,
                "type": "synthetic-data",
                "info": {
                    "name": "My awesome block",
                    "description": "Don't overwrite dis"
                },
                "parameters": [
                    {
                        "name": "OpenAI API Key",
                        "value": "",
                        "type": "secret",
                        "help": "An API Key that gives access to OpenAI",
                        "param": "OPENAI_API_KEY"
                    }
                ],
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, '.ei-block-config'), 'utf-8')), {
                "version": 2,
                "config": {
                    "211.212.213.214": {
                        "organizationId": 1,
                        "id": 7174
                    }
                }
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, 'parameters.json'), 'utf-8')), {
                "version": 1,
                "type": "synthetic-data",
                "info": {
                    "name": "My awesome block",
                    "description": "Don't overwrite dis"
                },
                "parameters": [
                    {
                        "name": "OpenAI API Key",
                        "value": "",
                        "type": "secret",
                        "help": "An API Key that gives access to OpenAI",
                        "param": "OPENAI_API_KEY"
                    },
                ],
            });
        });
        it('loads v2 synthetic data (same host, new parameters.json)', async () => {
            let config = {};
            config.host = '211.212.213.214';
            let folder = await fs_1.default.promises.mkdtemp(path_1.default.join(os_1.default.tmpdir() + '-ei-'));
            let eiBlockConfig = {
                "version": 2,
                "config": {
                    "211.212.213.214": {
                        "organizationId": 1,
                        "id": 7174
                    }
                }
            };
            await fs_1.default.promises.writeFile(path_1.default.join(folder, '.ei-block-config'), JSON.stringify(eiBlockConfig, null, 4), 'utf-8');
            await fs_1.default.promises.writeFile(path_1.default.join(folder, 'parameters.json'), JSON.stringify({
                "version": 1,
                "type": "synthetic-data",
                "info": {
                    "name": "My awesome block",
                    "description": "Don't overwrite dis"
                },
                "parameters": [
                    {
                        "name": "OpenAI API Key",
                        "value": "",
                        "type": "secret",
                        "help": "An API Key that gives access to OpenAI",
                        "param": "OPENAI_API_KEY"
                    },
                ]
            }, null, 4), 'utf-8');
            const blockConfigManager = new block_config_manager_1.BlockConfigManager(config, folder, {
                skipConfirmation: true,
            });
            const blockConfig = await blockConfigManager.loadConfig({ throwOnMissingParams: false });
            (0, assert_1.default)(blockConfig !== null, 'blockConfig should not be null');
            if (blockConfig.type !== 'synthetic-data') {
                (0, assert_1.default)(false, 'blockConfig type should be "synthetic-data": ' + JSON.stringify(blockConfig));
            }
            assert_1.default.deepEqual(blockConfig.type, 'synthetic-data');
            assert_1.default.deepEqual(blockConfig.config, {
                "organizationId": 1,
                "id": 7174
            });
            assert_1.default.deepEqual(blockConfig.parameters, {
                "version": 1,
                "type": "synthetic-data",
                "info": {
                    "name": "My awesome block",
                    "description": "Don't overwrite dis"
                },
                "parameters": [
                    {
                        "name": "OpenAI API Key",
                        "value": "",
                        "type": "secret",
                        "help": "An API Key that gives access to OpenAI",
                        "param": "OPENAI_API_KEY"
                    }
                ],
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, '.ei-block-config'), 'utf-8')), {
                "version": 2,
                "config": {
                    "211.212.213.214": {
                        "organizationId": 1,
                        "id": 7174
                    }
                }
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, 'parameters.json'), 'utf-8')), {
                "version": 1,
                "type": "synthetic-data",
                "info": {
                    "name": "My awesome block",
                    "description": "Don't overwrite dis"
                },
                "parameters": [
                    {
                        "name": "OpenAI API Key",
                        "value": "",
                        "type": "secret",
                        "help": "An API Key that gives access to OpenAI",
                        "param": "OPENAI_API_KEY"
                    },
                ],
            });
        });
    });
    describe('transformation', () => {
        it('converts v1 transform block (other host, no parameters.json)', async () => {
            let config = {};
            config.host = '211.212.213.214';
            config.api = {
                organizationBlocks: {
                    getOrganizationTransformationBlock: async (organizationId, blockId) => {
                        return {
                            success: true,
                            transformationBlock: {
                                id: blockId,
                                showInSyntheticData: false,
                            }
                        };
                    },
                },
            };
            let folder = await fs_1.default.promises.mkdtemp(path_1.default.join(os_1.default.tmpdir() + '-ei-'));
            let eiBlockConfig = {
                "version": 1,
                "config": {
                    "edgeimpulse.com": {
                        "name": "Mix in noise",
                        "type": "transform",
                        "description": "Takes WAV files in, and adds noise to them",
                        "organizationId": 1,
                        "operatesOn": "file",
                        "tlIndRequiresGpu": false,
                        "transformMountpoints": [],
                        "id": 155
                    }
                }
            };
            await fs_1.default.promises.writeFile(path_1.default.join(folder, '.ei-block-config'), JSON.stringify(eiBlockConfig, null, 4), 'utf-8');
            const blockConfigManager = new block_config_manager_1.BlockConfigManager(config, folder, {
                skipConfirmation: true,
                transformWhatTypeOfBlockIsThisReply: 'transform',
            });
            const blockConfig = await blockConfigManager.loadConfig({ throwOnMissingParams: false });
            (0, assert_1.default)(blockConfig !== null, 'blockConfig should not be null');
            if (blockConfig.type !== 'transform') {
                (0, assert_1.default)(false, 'blockConfig type should be "transform": ' + JSON.stringify(blockConfig));
            }
            assert_1.default.deepEqual(blockConfig.type, 'transform');
            assert_1.default.deepEqual(blockConfig.config, null); // host is different
            assert_1.default.deepEqual(blockConfig.parameters, {
                "version": 1,
                "type": "transform",
                "info": {
                    "name": "Mix in noise",
                    "description": "Takes WAV files in, and adds noise to them",
                    "operatesOn": "file",
                    "transformMountpoints": [],
                },
                "parameters": []
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, '.ei-block-config'), 'utf-8')), {
                "version": 2,
                "config": {
                    "edgeimpulse.com": {
                        "organizationId": 1,
                        "id": 155
                    }
                }
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, 'parameters.json'), 'utf-8')), {
                "version": 1,
                "type": "transform",
                "info": {
                    "name": "Mix in noise",
                    "description": "Takes WAV files in, and adds noise to them",
                    "operatesOn": "file",
                    "transformMountpoints": [],
                },
                "parameters": []
            });
        });
        it('converts v1 transform block (same host, no parameters.json)', async () => {
            let config = {};
            config.host = '211.212.213.214';
            config.api = {
                organizationBlocks: {
                    getOrganizationTransformationBlock: async (organizationId, blockId) => {
                        return {
                            success: true,
                            transformationBlock: {
                                id: blockId,
                                showInSyntheticData: false,
                            }
                        };
                    },
                },
            };
            let folder = await fs_1.default.promises.mkdtemp(path_1.default.join(os_1.default.tmpdir() + '-ei-'));
            let eiBlockConfig = {
                "version": 1,
                "config": {
                    "211.212.213.214": {
                        "name": "Mix in noise",
                        "type": "transform",
                        "description": "Takes WAV files in, and adds noise to them",
                        "organizationId": 1,
                        "operatesOn": "file",
                        "tlIndRequiresGpu": false,
                        "transformMountpoints": [],
                        "id": 155
                    }
                }
            };
            await fs_1.default.promises.writeFile(path_1.default.join(folder, '.ei-block-config'), JSON.stringify(eiBlockConfig, null, 4), 'utf-8');
            const blockConfigManager = new block_config_manager_1.BlockConfigManager(config, folder, {
                skipConfirmation: true,
            });
            const blockConfig = await blockConfigManager.loadConfig({ throwOnMissingParams: false });
            (0, assert_1.default)(blockConfig !== null, 'blockConfig should not be null');
            if (blockConfig.type !== 'transform') {
                (0, assert_1.default)(false, 'blockConfig type should be "transform": ' + JSON.stringify(blockConfig));
            }
            assert_1.default.deepEqual(blockConfig.type, 'transform');
            assert_1.default.deepEqual(blockConfig.config, {
                "organizationId": 1,
                "id": 155
            });
            assert_1.default.deepEqual(blockConfig.parameters, {
                "version": 1,
                "type": "transform",
                "info": {
                    "name": "Mix in noise",
                    "description": "Takes WAV files in, and adds noise to them",
                    "operatesOn": "file",
                    "transformMountpoints": [],
                },
                "parameters": []
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, '.ei-block-config'), 'utf-8')), {
                "version": 2,
                "config": {
                    "211.212.213.214": {
                        "organizationId": 1,
                        "id": 155
                    }
                }
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, 'parameters.json'), 'utf-8')), {
                "version": 1,
                "type": "transform",
                "info": {
                    "name": "Mix in noise",
                    "description": "Takes WAV files in, and adds noise to them",
                    "operatesOn": "file",
                    "transformMountpoints": [],
                },
                "parameters": []
            });
        });
        it('converts v1 transform block (same host, old parameters.json)', async () => {
            let config = {};
            config.host = '211.212.213.214';
            config.api = {
                organizationBlocks: {
                    getOrganizationTransformationBlock: async (organizationId, blockId) => {
                        return {
                            success: true,
                            transformationBlock: {
                                id: blockId,
                                showInSyntheticData: false,
                            }
                        };
                    },
                },
            };
            let folder = await fs_1.default.promises.mkdtemp(path_1.default.join(os_1.default.tmpdir() + '-ei-'));
            let eiBlockConfig = {
                "version": 1,
                "config": {
                    "211.212.213.214": {
                        "name": "Mix in noise",
                        "type": "transform",
                        "description": "Takes WAV files in, and adds noise to them",
                        "organizationId": 1,
                        "operatesOn": "file",
                        "tlIndRequiresGpu": false,
                        "transformMountpoints": [],
                        "id": 155
                    },
                }
            };
            await fs_1.default.promises.writeFile(path_1.default.join(folder, '.ei-block-config'), JSON.stringify(eiBlockConfig, null, 4), 'utf-8');
            await fs_1.default.promises.writeFile(path_1.default.join(folder, 'parameters.json'), JSON.stringify([
                {
                    "name": "Number of files to create",
                    "help": "How many new files to create per input file. Noise is randomly mixed in per file.",
                    "type": "int",
                    "value": "10",
                    "param": "out-count"
                }
            ], null, 4), 'utf-8');
            const blockConfigManager = new block_config_manager_1.BlockConfigManager(config, folder, {
                skipConfirmation: true,
            });
            const blockConfig = await blockConfigManager.loadConfig({ throwOnMissingParams: false });
            (0, assert_1.default)(blockConfig !== null, 'blockConfig should not be null');
            if (blockConfig.type !== 'transform') {
                (0, assert_1.default)(false, 'blockConfig type should be "transform": ' + JSON.stringify(blockConfig));
            }
            assert_1.default.deepEqual(blockConfig.type, 'transform');
            assert_1.default.deepEqual(blockConfig.config, {
                "organizationId": 1,
                "id": 155
            });
            assert_1.default.deepEqual(blockConfig.parameters, {
                "version": 1,
                "type": "transform",
                "info": {
                    "name": "Mix in noise",
                    "description": "Takes WAV files in, and adds noise to them",
                    "operatesOn": "file",
                    "transformMountpoints": [],
                },
                "parameters": [
                    {
                        "name": "Number of files to create",
                        "help": "How many new files to create per input file. Noise is randomly mixed in per file.",
                        "type": "int",
                        "value": "10",
                        "param": "out-count"
                    },
                ],
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, '.ei-block-config'), 'utf-8')), {
                "version": 2,
                "config": {
                    "211.212.213.214": {
                        "organizationId": 1,
                        "id": 155
                    }
                }
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, 'parameters.json'), 'utf-8')), {
                "version": 1,
                "type": "transform",
                "info": {
                    "name": "Mix in noise",
                    "description": "Takes WAV files in, and adds noise to them",
                    "operatesOn": "file",
                    "transformMountpoints": [],
                },
                "parameters": [
                    {
                        "name": "Number of files to create",
                        "help": "How many new files to create per input file. Noise is randomly mixed in per file.",
                        "type": "int",
                        "value": "10",
                        "param": "out-count"
                    },
                ],
            });
        });
        it('converts v1 transform block (same host, new parameters.json)', async () => {
            let config = {};
            config.host = '211.212.213.214';
            config.api = {
                organizationBlocks: {
                    getOrganizationTransformationBlock: async (organizationId, blockId) => {
                        return {
                            success: true,
                            transformationBlock: {
                                id: blockId,
                                showInSyntheticData: false,
                            }
                        };
                    },
                },
            };
            let folder = await fs_1.default.promises.mkdtemp(path_1.default.join(os_1.default.tmpdir() + '-ei-'));
            let eiBlockConfig = {
                "version": 1,
                "config": {
                    "211.212.213.214": {
                        "name": "Mix in noise",
                        "type": "transform",
                        "description": "Takes WAV files in, and adds noise to them",
                        "organizationId": 1,
                        "operatesOn": "file",
                        "tlIndRequiresGpu": false,
                        "transformMountpoints": [{ "bleep": 123 }],
                        "id": 155
                    },
                }
            };
            await fs_1.default.promises.writeFile(path_1.default.join(folder, '.ei-block-config'), JSON.stringify(eiBlockConfig, null, 4), 'utf-8');
            await fs_1.default.promises.writeFile(path_1.default.join(folder, 'parameters.json'), JSON.stringify({
                "version": 1,
                "type": "transform",
                "info": {
                    "name": "My awesome block",
                    "description": "Don't overwrite dis",
                    "operatesOn": "directory"
                },
                "parameters": [
                    {
                        "name": "Number of files to create #2",
                        "help": "How many new files to create per input file. Noise is randomly mixed in per file.",
                        "type": "int",
                        "value": "10",
                        "param": "out-count"
                    },
                ]
            }, null, 4), 'utf-8');
            const blockConfigManager = new block_config_manager_1.BlockConfigManager(config, folder, {
                skipConfirmation: true,
            });
            const blockConfig = await blockConfigManager.loadConfig({ throwOnMissingParams: false });
            (0, assert_1.default)(blockConfig !== null, 'blockConfig should not be null');
            if (blockConfig.type !== 'transform') {
                (0, assert_1.default)(false, 'blockConfig type should be "transform": ' + JSON.stringify(blockConfig));
            }
            assert_1.default.deepEqual(blockConfig.type, 'transform');
            assert_1.default.deepEqual(blockConfig.config, {
                "organizationId": 1,
                "id": 155
            });
            assert_1.default.deepEqual(blockConfig.parameters, {
                "version": 1,
                "type": "transform",
                "info": {
                    "name": "My awesome block",
                    "description": "Don't overwrite dis",
                    "operatesOn": "directory", // should not be overwritten
                    // shouldn't have mountPoints here (if already this format shouldn't read from .ei-block-config)
                },
                "parameters": [
                    {
                        "name": "Number of files to create #2",
                        "help": "How many new files to create per input file. Noise is randomly mixed in per file.",
                        "type": "int",
                        "value": "10",
                        "param": "out-count"
                    },
                ]
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, '.ei-block-config'), 'utf-8')), {
                "version": 2,
                "config": {
                    "211.212.213.214": {
                        "organizationId": 1,
                        "id": 155
                    }
                }
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, 'parameters.json'), 'utf-8')), {
                "version": 1,
                "type": "transform",
                "info": {
                    "name": "My awesome block",
                    "description": "Don't overwrite dis",
                    "operatesOn": "directory", // should not be overwritten
                    // shouldn't have mountPoints here (if already this format shouldn't read from .ei-block-config)
                },
                "parameters": [
                    {
                        "name": "Number of files to create #2",
                        "help": "How many new files to create per input file. Noise is randomly mixed in per file.",
                        "type": "int",
                        "value": "10",
                        "param": "out-count"
                    },
                ]
            });
        });
        it('loads v2 transform block (same host, new parameters.json)', async () => {
            let config = {};
            config.host = '211.212.213.214';
            let folder = await fs_1.default.promises.mkdtemp(path_1.default.join(os_1.default.tmpdir() + '-ei-'));
            let eiBlockConfig = {
                "version": 2,
                "config": {
                    "211.212.213.214": {
                        "organizationId": 1,
                        "id": 155
                    },
                }
            };
            await fs_1.default.promises.writeFile(path_1.default.join(folder, '.ei-block-config'), JSON.stringify(eiBlockConfig, null, 4), 'utf-8');
            await fs_1.default.promises.writeFile(path_1.default.join(folder, 'parameters.json'), JSON.stringify({
                "version": 1,
                "type": "transform",
                "info": {
                    "name": "My awesome block",
                    "description": "Don't overwrite dis",
                    "operatesOn": "directory"
                },
                "parameters": [
                    {
                        "name": "Number of files to create #2",
                        "help": "How many new files to create per input file. Noise is randomly mixed in per file.",
                        "type": "int",
                        "value": "10",
                        "param": "out-count"
                    },
                ]
            }, null, 4), 'utf-8');
            const blockConfigManager = new block_config_manager_1.BlockConfigManager(config, folder, {
                skipConfirmation: true,
            });
            const blockConfig = await blockConfigManager.loadConfig({ throwOnMissingParams: false });
            (0, assert_1.default)(blockConfig !== null, 'blockConfig should not be null');
            if (blockConfig.type !== 'transform') {
                (0, assert_1.default)(false, 'blockConfig type should be "transform": ' + JSON.stringify(blockConfig));
            }
            assert_1.default.deepEqual(blockConfig.type, 'transform');
            assert_1.default.deepEqual(blockConfig.config, {
                "organizationId": 1,
                "id": 155
            });
            assert_1.default.deepEqual(blockConfig.parameters, {
                "version": 1,
                "type": "transform",
                "info": {
                    "name": "My awesome block",
                    "description": "Don't overwrite dis",
                    "operatesOn": "directory", // should not be overwritten
                    // shouldn't have mountPoints here (if already this format shouldn't read from .ei-block-config)
                },
                "parameters": [
                    {
                        "name": "Number of files to create #2",
                        "help": "How many new files to create per input file. Noise is randomly mixed in per file.",
                        "type": "int",
                        "value": "10",
                        "param": "out-count"
                    },
                ]
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, '.ei-block-config'), 'utf-8')), {
                "version": 2,
                "config": {
                    "211.212.213.214": {
                        "organizationId": 1,
                        "id": 155
                    }
                }
            });
            assert_1.default.deepStrictEqual(JSON.parse(await fs_1.default.promises.readFile(path_1.default.join(folder, 'parameters.json'), 'utf-8')), {
                "version": 1,
                "type": "transform",
                "info": {
                    "name": "My awesome block",
                    "description": "Don't overwrite dis",
                    "operatesOn": "directory", // should not be overwritten
                    // shouldn't have mountPoints here (if already this format shouldn't read from .ei-block-config)
                },
                "parameters": [
                    {
                        "name": "Number of files to create #2",
                        "help": "How many new files to create per input file. Noise is randomly mixed in per file.",
                        "type": "int",
                        "value": "10",
                        "param": "out-count"
                    },
                ]
            });
        });
    });
});
//# sourceMappingURL=transform.test.js.map